@@ -130,7 +130,7 @@ def get_datasets(ds_master, ds_paths_master, ds_behav_master=None, warnings=True
                 continue
             assert ds_name in DSs.keys(),\
                 print(f"\n\033[31;1mWARNING dataset {ds_name} from \033[34;1mds_paths_master\033[31;1m isn't referenced in \033[34;1mds_master\033[31;1m.")
-            DSs[ds_name]["dp"]=str(dp)
+            DSs[ds_name]["dp"]=dp
             for prb in DSs[ds_name].keys():
                 if 'probe' in prb:
                     dp_prb=dp/f'{ds_name}_{prb}'
@@ -139,7 +139,7 @@ def get_datasets(ds_master, ds_paths_master, ds_behav_master=None, warnings=True
                         f"Edit path of {ds_name}:path in \033[34;1mds_paths_master\033[31;1m "
                         "and check that all probes are in subdirectories."))
                         continue
-                    DSs[ds_name][prb]["dp"]=str(dp_prb)
+                    DSs[ds_name][prb]["dp"]=dp_prb
 
     for ds_name, ds in DSs.items():
         if "dp" not in ds.keys():
@@ -155,6 +155,46 @@ def get_datasets(ds_master, ds_paths_master, ds_behav_master=None, warnings=True
 
     return DSs
 
+def make_connected_pairs_df(ds_master, ds_paths_master, ds_behav_master):
+    
+    DSs = get_datasets(ds_master, ds_paths_master, ds_behav_master, warnings=False)
+    connected_pairs_df=pd.DataFrame(columns=['ds','dp', 'ss','cs','nc', 'holds_no_behav', 'holds_no_sync'])
+    for dsname, ds in DSs.items():
+        for dsk, dsv in ds.items():
+            if 'probe' in dsk:
+                probei= 0 # if merged dataset: int(dsk[-1])-1
+                if 'ss_cnc_real' in dsv.keys():
+                    if np.any(dsv['ss_cnc_real']):
+                        dp_dic = {probe:str(ds[probe]['dp']) for probe in ds.keys() if 'probe' in probe}
+                        dp, ds_table = merge_datasets(dp_dic)
+                        
+                        putative_pairs = [str(pair) for pair in dsv["ss_cnc_put"]]
+                        assert len(putative_pairs)>0, f"No connected pairs found in dataset {dsname} at {ds_master}!"
+                        real_pairs = [str(pair) for pair in dsv["ss_cnc_real"]]
+                        assert np.all(np.isin(real_pairs, putative_pairs)),\
+                            f"Some real connected pairs are not in putative connected pairs: {real_pairs[~np.isin(real_pairs, putative_pairs)]}.\n Edit json file at {ds_master}!"
+                        
+                        for ss,cnc in dsv["ss_cnc_put"]:
+                            
+                            cs=np.nan
+                            if np.any(dsv['ss_cs']):
+                                ss_cs = npa(dsv['ss_cs'])
+                                if ss in ss_cs[:,0]:
+                                    cs = ss_cs[np.isin(ss_cs[:,0],[ss]),1][0]
+                                    
+                            if str([ss,cnc]) in real_pairs:
+                                holds_no_sync = True
+                            else:
+                                holds_no_sync = False
+                                
+                            holds_no_behav = True ## TODO
+                            connected_pairs_df=connected_pairs_df.append(
+                                {'ds':dsname, 'dp':dp,
+                                 'ss':ss+0.1*probei, 'cs':cs+0.1*probei, 'nc':cnc+0.1*probei,
+                                'holds_no_behav':holds_no_behav, 'holds_no_sync':holds_no_sync},
+                                 ignore_index=True)
+    return connected_pairs_df
+
 def get_rec_len(dp, unit='seconds'):
     ' returns recording length in seconds or samples'
     assert unit in ['samples', 'seconds']
@@ -199,7 +239,7 @@ def detect_new_spikesorting(dp, print_message=True, qualities=None):
         if not np.all(np.isin(old_clusters,new_clusters)): spikesorted = True
 
     if spikesorted and print_message:
-        print('\n\033[34;1m--- New spike-sorting detected.')
+        print('\n\033[34;1m--- New spike-sorting detected.\033[0m')
 
     return spikesorted
 
@@ -283,7 +323,7 @@ def load_merged_units_qualities(dp_merged, ds_table=None):
 
 def get_units(dp, quality='all', chan_range=None, again=False):
 
-    assert quality in ['all', 'good', 'mua', 'noise']
+    assert quality in ['all', 'good', 'mua', 'noise', 'unsorted']
 
     if assert_multi(dp):
         qualities = load_merged_units_qualities(dp)
@@ -323,9 +363,11 @@ def get_good_units(dp):
 def check_periods(periods):
     err_mess = "periods can only be 'all' or a list of lists/tuples [[t1.1,t1.2], [t2.1,t2.2]...] in seconds!"
     if isinstance(periods, str):
-        assert periods=='all', err_mess
+        assert periods == 'all', err_mess
         return periods
     periods = npa(periods)
+    if periods.ndim==1:
+        periods = periods.reshape(1,-1)
     assert periods.ndim == 2, "When feeding a single period [t1,t2], do not forget the outer brackets [[t1,t2]]!"
     assert periods.shape[1] == 2, err_mess
     assert np.all(np.diff(periods, axis=1)>0),\
@@ -336,4 +378,4 @@ def check_periods(periods):
 # circular imports
 from npyx.inout import read_metadata
 from npyx.spk_wvf import get_depthSort_peakChans
-from npyx.merger import assert_multi, get_ds_table
+from npyx.merger import assert_multi, get_ds_table, merge_datasets
