@@ -9,7 +9,8 @@ import json
 import h5py
 
 from npyx.utils import assert_int, assert_float
-from npyx.inout import get_binary_file_path, get_npix_sync, chan_map, extract_rawChunk, read_metadata, preprocess_binary_file
+from npyx.inout import get_binary_file_path, get_npix_sync, chan_map,\
+                       extract_rawChunk, read_metadata, preprocess_binary_file, detect_hardware_filter
 from npyx.spk_t import ids, trn, trn_filtered
 from npyx.spk_wvf import wvf_dsmatch
 from npyx.gl import get_units, check_periods
@@ -62,14 +63,48 @@ def add_units_to_h5(h5_path, dp, units=None, **kwargs):
     for u in units:
         add_unit_h5(h5_path, dp, u, **kwargs)
 
+
+def relative_unit_path_h5(dataset, unit):
+    return f"datasets/{dataset}/{unit}"
+
+
+def get_unit_paths_h5(h5_file, dataset, unit,
+                      lab_id = 'hausser', unit_absolute_id = None):
+    relative_unit_path = relative_unit_path_h5(dataset, unit)
+    if relative_unit_path in h5_file:
+            absolute_unit_path = h5_file[f'{relative_unit_path}/neuron_absolute_id'][()].decode()
+    else:
+        if unit_absolute_id is None:
+            if f"{lab_id}_neuron_0" not in h5_file:
+                unit_absolute_id = 0
+            else:
+                root_groups = list(h5_file.keys())
+                neuron_ids = [int(x.split('_')[-1]) for x in root_groups if f"{lab_id}_neuron" in x]
+                unit_absolute_id = np.sort(neuron_ids)[-1] + 1
+        absolute_unit_path = f'{lab_id}_neuron_{unit_absolute_id}'
+            
+    return relative_unit_path, absolute_unit_path
+
+
+def remove_unit_h5(h5_path, dp, unit, lab_id='hausser'):
+    dataset = Path(dp).name
+    with h5py.File(h5_path, "a") as h5_file:
+        relative_unit_path, absolute_unit_path = get_unit_paths_h5(h5_file, dataset, unit, lab_id)
+        del h5_file[relative_unit_path]
+        del h5_file[absolute_unit_path]
+        dataset_path = str(Path(relative_unit_path).parent)
+        if len(h5_file[dataset_path].keys()) == 0:
+            del h5_file[dataset_path]
+
+
 def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
-                unit_abolute_id=None, sync_chan_id=None,
+                sync_chan_id=None,
                 again=False, again_wvf=False, plot_debug=False, verbose=False,
                 dataset=None,
                 raw_window=[0.1, 30.1], center_raw_window_on_spikes=True,
                 include_raw_snippets=True, include_whitened_snippets=True,
                 raw_snippet_halfrange=2, mean_wvf_half_range=11,
-                sane_spikes=None, sane_before_opto=False, include_fp_fn_mask=True,
+                sane_spikes=None, sane_periods=None, sane_before_opto=False, include_fp_fn_mask=True,
                 optostims=None, optostims_from_sync=False, optostims_threshold=None,
                 n_waveforms_for_matching=5000,
                 **kwargs):
@@ -82,7 +117,7 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
      ---->>> data format details here www.tinyurl.com/c4database <<<---
 
     Each unit can be accessed from 2 paths which point to the same data:
-    - an absolute path, {unit_abolute_id}/
+    - an absolute path, {unit_absolute_id}/
                         which allows for a flat hierarchy and makes it easier to compute features,
                         easier to work on data from other labs
     - a relative path, datasets/{dataset}/{unit}/
@@ -121,6 +156,7 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
     
     - sane_spikes: optional bool array, custom definition of 'sane spikes' for whatever reason.
                    By default, will be all spikes within 'periods'.
+    - sane_periods: optional list of [start, stop] periods, in seconds, to define 'sane spikes'. Will prevail over "sane_before_opto" if both are defined.
     - sane_before_opto: bool, whether to consider all spikes before the 1st optostim
                         to constitute the sane_spikes boolean mask.
                         Will only work if optostims are provided (or alternatively optostims_from_sync is True.)
@@ -132,7 +168,7 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
                  (1st column: onsets, 2nd column: offsets).
                  By default None, will be read from sync channel (at sync_chan_id) if optostims_from_sync is True.
     - optostims_from_sync: bool, whether to pick up optostims from sync channel if None are provided.
-    - optostims_threshold: float, time before which optostims will be ignored
+    - optostims_threshold: float, time before which optostims will be ignored (same units as optostims, seconds if optostims_from_sync=True).
                            (handles sync signals without light at beginning of recording)
                            
     - n_waveforms_for_matching: int, number of waveforms to subsample for drift-shift matching
@@ -159,37 +195,29 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
     h5_path = Path(h5_path)
     assert_h5_file(h5_path)
     with h5py.File(h5_path, "a") as h5_file:
-
-        # check whether neuron already exists in dataset
+        
+        # format dataset name
         if dataset is None:
             dataset = dp.name
         check_dataset_format(dataset)
-        neuron_path = f"datasets/{dataset}/{unit}"
-        if neuron_path in h5_file:
-            neuron_absolute_path=h5_file[f'{neuron_path}/neuron_absolute_id'][()].decode()
+        
+        # Define or fetch unit h5 paths
+        relative_unit_path, absolute_unit_path = get_unit_paths_h5(h5_file, dataset, unit, lab_id)
+        if relative_unit_path in h5_file:
             if again:
-                del h5_file[neuron_path]
-                del h5_file[neuron_absolute_path]
+                del h5_file[relative_unit_path]
+                del h5_file[absolute_unit_path]
             else:
-                print(f"Neuron found in h5 file: {neuron_path} ({neuron_absolute_path})")
-                neuron_group = h5_file[neuron_absolute_path]
-                #return neuron_path
-            
-        if neuron_path not in h5_file or again:
-            # figure out where we're at
-            if unit_abolute_id is None:
-                if f"{lab_id}_neuron_0" not in h5_file:
-                    unit_abolute_id = 0
-                else:
-                    root_groups = list(h5_file.keys())
-                    neuron_ids = [int(x.split('_')[-1]) for x in root_groups if f"{lab_id}_neuron" in x]
-                    unit_abolute_id = np.sort(neuron_ids)[-1] + 1
-
-            # create group for new neuron
-            neuron_absolute_path=f'{lab_id}_neuron_{unit_abolute_id}'
-            neuron_group = h5_file.create_group(neuron_absolute_path)
-            h5_file[neuron_path] = neuron_group
-            print(f"Adding data at {neuron_path} ({neuron_absolute_path})...")
+                print(f"Neuron found in h5 file: {relative_unit_path} ({absolute_unit_path})")
+                neuron_group = h5_file[absolute_unit_path]
+        # redefine unit paths in case the unit got deleted
+        #relative_unit_path, absolute_unit_path = get_unit_paths_h5(h5_file, dataset, unit, lab_id)
+        
+        # create group for new neuron if necessary
+        if relative_unit_path not in h5_file:
+            neuron_group = h5_file.create_group(absolute_unit_path)
+            h5_file[relative_unit_path] = neuron_group
+            print(f"Adding data at {relative_unit_path} ({absolute_unit_path})...")
 
         # metadata
         write_to_group(neuron_group, 'lab_id', lab_id, again)
@@ -208,7 +236,29 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
         if 'spike_indices' not in neuron_group or again:
             t = trn(dp, unit, periods=periods, again=again)
             write_to_group(neuron_group, 'spike_indices', t, again)
-            
+        else:
+            t = neuron_group['spike_indices']
+
+        # usable spikes mask
+        if 'sane_spikes' not in neuron_group or again:
+            if sane_spikes is None and\
+                sane_periods is not None:
+                sane_periods = check_periods(sane_periods)
+                sane_periods*=samp_rate
+                sane_spikes = (t*0).astype(bool)
+                for sane_period in sane_periods:
+                    sane_spikes = sane_spikes|(t>=sane_period[0])&(t<=sane_period[1])
+            elif sane_spikes is None and\
+                optostims is not None and\
+                sane_before_opto:
+                # Only consider spikes 10s before first opto onset
+                sane_spikes = (t < (optostims[0,0]-10)*samp_rate)
+            else:
+                sane_spikes = (t*0+1).astype(bool)
+            write_to_group(neuron_group, 'sane_spikes', sane_spikes, again)
+
+        # optostims
+        if 'optostims' not in neuron_group or again:
             if optostims is None and optostims_from_sync:
                 ons, offs = get_npix_sync(dp, verbose=False)
                 if sync_chan_id is None:
@@ -222,19 +272,9 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
             if optostims is not None and optostims_threshold is not None:
                 opto_m = optostims[:,0] > optostims_threshold
                 optostims = optostims[opto_m,:]
-                
-            if sane_spikes is None and\
-                optostims is not None and\
-                sane_before_opto:
-                # Only consider spikes 10s before first opto onset
-                sane_spikes = (t < (optostims[0,0]-10)*samp_rate)
-            else:
-                sane_spikes = (t*0+1).astype(bool)
-                
-        if optostims is None:
-            optostims = np.array([])
-        write_to_group(neuron_group, 'optostims', optostims, again)
-        write_to_group(neuron_group, 'sane_spikes', sane_spikes, again)
+
+            if optostims is not None:
+                write_to_group(neuron_group, 'optostims', optostims, again)
 
         # waveforms
         k = ['mean_waveform_preprocessed', 'amplitudes', 'voltage_sample', 'peakchan_SNR']
@@ -242,7 +282,8 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
             # must recompute chan_bottom and chan_top - suboptimal, can be rewritten
             dsm_tuple = wvf_dsmatch(dp, unit, t_waveforms=waveform_samples, periods=periods,
                                     again=again_wvf, plot_debug=plot_debug, verbose=verbose,
-                                    n_waves_used_for_matching=n_waveforms_for_matching)
+                                    n_waves_used_for_matching=n_waveforms_for_matching, 
+                                    med_sub = True, nRangeMedSub=None)
             dsm_waveform, peak_chan = dsm_tuple[1], dsm_tuple[3]
             write_to_group(neuron_group, 'primary_channel', peak_chan)
             chan_range = np.arange(peak_chan-mean_wvf_half_range, peak_chan+mean_wvf_half_range)
@@ -258,14 +299,14 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
             write_to_group(neuron_group, 'channel_ids',
                            np.arange(chan_bottom, chan_top, dtype=np.dtype('uint16')),
                            again)
-            write_to_group(neuron_group, 'channelmap', cm[chan_bottom:chan_top, 1:2], again)
+            write_to_group(neuron_group, 'channelmap', cm[chan_bottom:chan_top, 1:], again)
 
         # Extract voltage snippets
         k = ['amplitudes', 'channel_noise_std', 'peakchan_SNR', 'voltage_sample']
         if not all_keys_in_group(k, neuron_group)\
             or again:
             if center_raw_window_on_spikes:
-                t = h5_file[neuron_path+'/spike_indices'][...]/samp_rate
+                t = h5_file[relative_unit_path+'/spike_indices'][...]/samp_rate
                 if raw_window[1]>t[0]: # spike starting after end of original window
                     raw_window = np.array(raw_window)+t[0]
                     raw_window[1]=min(raw_window[1], t[-1])
@@ -292,7 +333,7 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
         if ('fn_fp_filtered_spikes' not in neuron_group or again) and include_fp_fn_mask:
             # get good spikes mask for all spikes
             # because trn_filtered can only work on a contiguous chunk
-            if periods is 'all':
+            if isinstance(periods, str): # can only be 'all', given check_periods
                 periods_m_range = [0, meta['recording_length_seconds']/60]
             else:
                 periods_m_range = [periods.min()/60, periods.max()/60]
@@ -301,7 +342,7 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
 
 
             # if periods is not all, trim down the mask to spikes in periods
-            if periods is not 'all':
+            if not isinstance(periods, str): # if str, can only be 'all', given check_periods
                 t = trn(dp, unit, periods=periods) # if again, as recomputed just above anyway, so don't pass the argument
                 t_all = trn(dp, unit) # grab all spikes
                 periods_mask = np.isin(t_all, t)
@@ -325,7 +366,7 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
         if ('whitened_voltage_sample' not in neuron_group or again)\
             and include_whitened_snippets:
             if center_raw_window_on_spikes:
-                t = h5_file[neuron_path+'/spike_indices'][...]/samp_rate
+                t = h5_file[relative_unit_path+'/spike_indices'][...]/samp_rate
                 if raw_window[1]>t[0]: # spike starting after end of original window
                     raw_window = np.array(raw_window)+t[0]
                     raw_window[1]=min(raw_window[1], t[-1])
@@ -361,7 +402,7 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
         write_to_group(neuron_group, 'hausser_label', 0, again)
         write_to_group(neuron_group, 'medina_label', 0, again)
 
-    return neuron_path
+    return relative_unit_path
 
 def load_json_datasets(json_path, include_missing_datasets=False):
     with open(json_path) as f:
@@ -386,43 +427,68 @@ def load_json_datasets(json_path, include_missing_datasets=False):
         ss = list(ds['ss'])
         cs = list(ds['cs'])
         print(f"Dataset {dp} found with opto responsive units {units}, simple spikes {ss}, complex spikes {cs}.\n")
-
-        units_m=np.isin(units+ss+cs, get_units(dp))
-        assert all(units_m), f"Units {np.array(units)[~units_m]} not found in {dp}!"
+        all_units = units+ss+cs
+        units_m=np.isin(all_units, get_units(dp))
+        assert all(units_m), f"Units {np.array(all_units)[~units_m]} not found in {dp}!"
 
     return DSs
 
 
-def add_json_datasets_to_h5(json_path, h5_path, preprocess_binary = True, lab_id="hausser",
-                            delete_original_data=False, data_deletion_double_check=False):
+def add_json_datasets_to_h5(json_path, h5_path, lab_id, preprocess_if_raw=False,
+                            delete_original_data=False, data_deletion_double_check=False,
+                            again=False, include_raw_snippets=False, verbose=False,
+                            include_all_good = False, **kwargs):
 
     DSs = load_json_datasets(json_path, include_missing_datasets=False)
 
     for ds_name, ds in DSs.items():
         dp=Path(ds['dp'])
 
-        if preprocess_binary:
-            preprocess_binary_file(dp,
-                delete_original_data=delete_original_data,
-                data_deletion_double_check=data_deletion_double_check)
+        if preprocess_if_raw:
+            if not detect_hardware_filter(dp):
+                print("\033[34;1mRaw file detected - filtering with 1st order butterworth highpass at 300Hz...\033[0m")
+                preprocess_binary_file(dp,
+                    delete_original_data=delete_original_data,
+                    data_deletion_double_check=data_deletion_double_check,
+                    median_subtract=False, filter_forward=True, filter_backward=False, order=1)
 
         optolabel=ds['ct']
         if optolabel=="PkC": optolabel="PkC_ss"
         units=ds['units']
         ss=ds['ss']
         cs=ds['cs']
-        for u in units+ss+cs:
-            add_unit_h5(h5_path, dp, u, lab_id=lab_id,
-                    unit_abolute_id=None, sync_chan_id=None,
-                    again=False, plot_debug=False, verbose=True)
+        good_units = list(get_units(dp, 'good'))
+        sane_times = ds["sane_times"] if "sane_times" in ds else None
+
+        if include_all_good:
+            units_for_h5 = np.unique(units+ss+cs+good_units)
+        else:
+            units_for_h5 = units+ss+cs
+
+        for u in units_for_h5:
+            add_unit_h5(h5_path, dp, u, lab_id, periods='all',
+                    again=again, again_wvf=again, verbose=verbose,
+                    include_raw_snippets=include_raw_snippets, include_whitened_snippets=include_raw_snippets,
+                    sane_periods=sane_times, **kwargs)
             if u in units:
                 label = optolabel
             elif u in ss:
                 label="PkC_ss"
             elif u in cs:
                 label="PkC_cs"
+            else:
+                continue
             label_optotagged_unit_h5(h5_path, ds_name, u, label)
 
+def add_json_datasets_to_h5_hausser(json_path, h5_path, again=False, include_raw_snippets=False,
+                                    delete_original_data=False, data_deletion_double_check=False,
+                                    include_all_good=False):
+
+    add_json_datasets_to_h5(json_path, h5_path, "hausser", preprocess_if_raw=False,
+                            delete_original_data=delete_original_data, data_deletion_double_check=data_deletion_double_check,
+                            again=again, include_raw_snippets=include_raw_snippets, verbose=False,
+                            optostims_from_sync=True, optostims_threshold=20*60, include_all_good=include_all_good)
+
 
 def add_data_to_unit_h5(h5_path, dataset, unit, data, field):
     """
@@ -435,10 +501,10 @@ def add_data_to_unit_h5(h5_path, dataset, unit, data, field):
     - field: name of dataset to add data (id exists already, will overwrite)
     """
     check_dataset_format(dataset)
-    neuron_path = f"datasets/{dataset}/{unit}"
+    unit_path = relative_unit_path_h5(dataset, unit)
     with h5py.File(h5_path, "a") as h5_file:
-        assert neuron_path in h5_file, f"WARNING unit {neuron_path} does not seem to be present in the file. To add it, use add_unit_h5()."
-        write_to_group(h5_file[neuron_path], field, data, True)
+        assert unit_path in h5_file, f"WARNING unit {unit_path} does not seem to be present in the file. To add it, use add_unit_h5()."
+        write_to_group(h5_file[unit_path], field, data, True)
 
 # h5 vizualisattion functions
 def print_h5_contents(h5_path, txt_output=False):
