@@ -84,7 +84,7 @@ def ids(dp, unit, sav=True, verbose=False, periods='all', again=False, enforced_
     # Optional selection of a section of the recording.
     # Always computed because cannot reasonably be part of file name.
     periods = check_periods(periods)
-    if periods is not 'all': # else, eq to periods=[(0, spike_samples[-1])] # in samples
+    if not isinstance(periods, str): # check_periods ensures that it should be 'all' if it is a string
         sec_bool=np.zeros(len(train), dtype=np.bool)
         for section in periods:
             sec_bool[(train>=section[0]*fs)&(train<=section[1]*fs)]=True # comparison in samples
@@ -166,7 +166,7 @@ def trn(dp, unit, sav=True, verbose=False, periods='all', again=False, enforced_
     # Optional selection of a section of the recording.
     # Always computed because cannot reasonably be part of file name.
     periods = check_periods(periods)
-    if not isinstance(periods, str):  # else, eq to periods=[(0, spike_samples[-1])] (in samples)
+    if not isinstance(periods, str): # check_periods ensures that it should be 'all' if it is a string
         sec_bool=np.zeros(len(train), dtype=np.bool)
         for section in periods:
             sec_bool[(train>=section[0]*fs)&(train<=section[1]*fs)]=True # comparison in samples
