@@ -2,11 +2,13 @@
 
 import os
 from pathlib import Path
+from tracemalloc import start
 
-from numba import jit, njit, prange
+from numba import njit
 from numba.typed import List
 from numba.core.errors import NumbaDeprecationWarning, NumbaPendingDeprecationWarning
 import warnings
+
 warnings.simplefilter("default", category=NumbaDeprecationWarning) #'ignore'
 warnings.simplefilter('default', category=NumbaPendingDeprecationWarning)#'ignore'
 
@@ -22,7 +24,6 @@ import scipy.signal as sgnl
 
 import logging
 from math import pi, log
-import pylab
 from scipy import fft, ifft
 from scipy.optimize import curve_fit
 from scipy.signal import cspline1d_eval, cspline1d
@@ -76,18 +77,19 @@ mark_dict = {
 
 #%% Utils
 
-_ACCEPTED_ARRAY_DTYPES = (np.float, np.float32, np.float64,
-                          np.int, np.int8, np.int16, np.uint8, np.uint16,
+_ACCEPTED_ARRAY_DTYPES = (np.float32, np.float64,
+                          np.int8, np.int16, np.uint8, np.uint16,
                           np.int32, np.int64, np.uint32, np.uint64,
-                          np.bool)
+                          bool)
 
 def assert_float(x):
-    return type(x) in [float, np.float, np.float16,
-                       np.float32, np.float64]
+    return isinstance(x, (float,
+                          np.float16, np.float32, np.float64))
 
 def assert_int(x):
-    return type(x) in [int, np.int, np.int8, np.int16, np.uint8,
-                       np.uint16, np.int32, np.int64, np.uint32, np.uint64]
+    return isinstance(x, (int, np.int8, np.int16,
+                          np.uint8, np.uint16, np.int32,
+                          np.int64, np.uint32, np.uint64))
 
 def assert_iterable(x):
     return hasattr(x, '__iter__')
@@ -111,6 +113,34 @@ def npa(arr=[], **kwargs):
         arr=np.array(arr, dtype=dtype)
     return arr
 
+def save_np_array(arr, filename, saveDir="~/Downloads", _format="npy", dtype=None):
+    """
+    Save a numpy array to a file.
+    
+    Parameters:
+        - arr: numpy array to save.
+        - filename: str, name of the file to save (without extension).
+        - saveDir: str, directory where to save the array.
+        - _format: 'npy' or 'npz'.
+        - dtype: numpy datatype. If provided, casts arrayt to this datatype before saving.
+    
+    Returns:
+        - None
+    """
+    saveDir = Path(saveDir).expanduser()
+    assert saveDir.exists(), f"{saveDir} does not exist!"
+
+    if dtype is not None:
+        arr = arr.astype(dtype)
+
+    putative_extension = filename[-4:]
+    if putative_extension not in [".npy", ".npz"]:
+        assert _format in ["npy", "npz"]
+        filename += f".{_format}"
+
+    np.save(saveDir/filename, arr)
+
+
 def isnumeric(x):
     x=str(x).replace('−','-')
     try:
@@ -156,8 +186,11 @@ def read_pyfile(filepath, ignored_chars=[" ", "'", "\"", "\n", "\r"]):
     return params
 
 def list_files(directory, extension, full_path=False):
+    """
+    List files with extension "extension" in directory "directory"."""
     directory=str(directory)
-    files = [f for f in os.listdir(directory) if f.endswith('.' + extension)]
+    if extension[0]!='.': extension = '.'+extension
+    files = [f for f in os.listdir(directory) if f.endswith(extension)]
     files.sort()
     if full_path:
         return [Path('/'.join([directory,f])) for f in files]
@@ -175,7 +208,7 @@ def any_n_consec(X, n_consec, where=False):
         - n_consec: int, number of consecutive True/False values to find
         - where: bool, returns array of indices if True
     '''
-    X=npa(X).astype(np.int)
+    X=npa(X).astype(np.int64)
     assert np.all((X==0)|(X==1)), 'X array should be only 0s and 1s!'
     for i in range(n_consec-1):
         X = X[:-1]+X[1:]
@@ -254,7 +287,7 @@ def thresh(arr, th, sgn=1, pos=1):
     assert sgn in [-1,1]
     arr=np.asarray(arr).copy()
     assert arr.ndim==1
-    arr= (arr-th)*sgn+th # Flips the array around threshold if sgn==-1
+    arr = (arr-th)*sgn+th # Flips the array around threshold if sgn==-1
 
     i=np.nonzero(arr>=th)[0] if pos==1 else np.nonzero(arr<=th)[0]
     # If no value is above the threshold or all of them are already above the threshold
@@ -359,69 +392,84 @@ def zscore(arr, frac=4./5, mn_ext=None, sd_ext=None):
     if sd==0: sd=1
     return (arr-mn)*1./sd
 
-def smooth(arr, method='gaussian_causal', sd=5, a=5, frac=0.06, it=0):
+def smooth(arr, method='gaussian_causal', sd=5, axis=1, gamma_a=5):
     '''
-    Smoothes a 1D array or a 2D array along axis 1
+    Smoothes a 1D array or a 2D array along specified axis.
     Parameters:
         - arr: ndarray/list, array to smooth
         - method: string, see methods implemented below | Default 'gaussian'
         - sd: int, gaussian window sd (in unit of array samples - e.g. use 10 for a 1ms std if bin size is 0.1ms) | Default 5
-        - a: sqrt of Gamma function rate (essentially std) | Default 5
-        - frac: lowess parameter, fraction watched ahead to perform smoothing | Default 0.06
-        - it: lowess paramater | Default 0
+        - axis: int axis along which smoothing is performed.
+        - a_gamma: sqrt of Gamma function rate (essentially std) | Default 5
 
     methods implemented:
         - gaussian
         - gaussian_causal
         - gamma (is causal)
-        - lowess
     '''
-    assert arr.ndim<=2
-    # Checks and formatting
-    assert method in ['gaussian', 'gaussian_causal', 'gamma', 'lowess']
-    assert type(sd) in [int, np.int]
-    arr=npa(arr).astype(float)
-    # pad with reversed CCG edges at the beginning and the end to prevnt edge effects...
-    C = arr.shape[-1]//2
-    if arr.ndim==1: padarr=np.hstack([arr[:C][::-1], arr, arr[-C:][::-1]])
-    elif arr.ndim==2: padarr=np.hstack([arr[:,:C][:,::-1], arr, arr[:,-C:][:,::-1]])
+    assert arr.ndim<=2,\
+        "WARNING this function runs on 3D arrays but seems to shift data leftwards - not functional yet."
+    if arr.ndim==1: axis=0
+    
+    ## Checks and formatting
+    assert method in ['gaussian', 'gaussian_causal', 'gamma']
+    assert type(sd) in [int, np.int32, np.int64]
 
-    # Compute the kernel
-    if method in ['gaussian', 'gaussian_causal', 'gamma']:
-        if method in ['gaussian', 'gaussian_causal']:
-            X=np.arange(-4*sd, 4*sd+1)
-            kernel=stt.norm.pdf(X, 0, sd)
-            if method=='gaussian_causal': kernel[:len(kernel)//2]=0
-        elif method=='gamma':
-            X=np.arange(a**2//2, max((a**2)*3//2+1, 10))
-            kernel=stt.gamma.pdf(X, a**2) # a = shape, b = scale = 1/rate. std=sqrt(a)/b = sqrt(a) for b=1
-        # WARNING the maximum should be centered to prevent data shift in time!
-        # This is achieved by padding the left of the kernel with zeros.
-        mx=np.nonzero(kernel==max(kernel))[0][0]
-        if mx<len(kernel)/2:
-            kernel=np.append(np.zeros(len(kernel)-2*mx), kernel)
-        elif mx>len(kernel)/2:
-            kernel=np.append(kernel, np.zeros(mx-(len(kernel)-mx)))
-        assert len(kernel)<padarr.shape[-1], 'The kernel is longer than the array to convolved, you must decrease the standard deviation parameter.'
 
-        # Convolve array with kernel
-        kernel=kernel/sum(kernel) # normalize kernel to prevent vertical scaling
-        if arr.ndim==1: sarr = np.convolve(padarr, kernel, mode='same')
-        elif arr.ndim==2: sarr = np.apply_along_axis(lambda m:np.convolve(m, kernel, mode='same'), axis=1, arr=padarr)
-        # kernel_offset=len(kernel)//2
-        # sarr=sarr[kernel_offset:-kernel_offset]
-    elif method=='lowess':
-        if arr.ndim==1: sarr = lowess(padarr, np.arange(padarr.shape[-1]), is_sorted=True, frac=frac, it=it)[:,1].flatten()
-        elif arr.ndim==2: sarr = np.apply_along_axis(lambda m:lowess(m, np.arange(len(m)), is_sorted=True, frac=frac, it=it), axis=1, arr=padarr)
+    ## pad array at beginning and end to prevent edge artefacts
+    C = arr.shape[axis]//2
+    pad_width = [[C,C] if i==axis else [0,0] for i in range(arr.ndim)]
+    padarr=np.pad(arr, pad_width, 'symmetric')
 
-    # Remove padding
-    if arr.ndim==1: sarr = sarr[C:-C]
-    elif arr.ndim==2:  sarr = sarr[:,C:-C]
 
+    ## Compute the kernel
+    if method in ['gaussian', 'gaussian_causal']:
+        X=np.arange(-4*sd, 4*sd+1)
+        kernel=stt.norm.pdf(X, 0, sd)
+        if method=='gaussian_causal':
+            kernel[:len(kernel)//2]=0
+    elif method=='gamma':
+        # a = shape, b = scale = 1/rate. std=sqrt(a)/b = sqrt(a) for b=1
+        X=np.arange(gamma_a**2//2, max((gamma_a**2)*3//2+1, 10))
+        kernel=stt.gamma.pdf(X, gamma_a**2)
+    
+    # center the maximum to prevent data shift in time
+    # This is achieved by padding the left/right of the kernel with zeros.
+    mx=np.argmax(kernel)
+    if mx<len(kernel)/2:
+        kernel=np.append(np.zeros(len(kernel)-2*mx), kernel)
+    elif mx>len(kernel)/2:
+        kernel=np.append(kernel, np.zeros(mx-(len(kernel)-mx)))
+    assert len(kernel)<padarr.shape[axis],\
+        'The kernel is longer than the array to convolved, you must decrease sd.'
+
+    # normalize kernel to prevent vertical scaling
+    kernel=kernel/sum(kernel)
+
+
+    ## Convolve array with kernel
+    sarr = np.apply_along_axis(lambda m:np.convolve(m, kernel, mode='same'), axis=axis, arr=padarr)
+
+
+    ## Remove padding
+    sarr = sarr[slice_along_axis(C,-C,axis=axis)]
     assert np.all(sarr.shape==arr.shape)
 
+
     return sarr
 
+
+def slice_along_axis(a,b,s=1,axis=0):
+    """
+    Returns properly formatted slice to slice array/list along specified axis.
+    - a: start
+    - b: end
+    - s: step
+    """
+    slc = slice(a,b,s)
+    return (slice(None),) * axis + (slc,)
+
+
 def xcorr_axis(x, y, axis=0):
     """
     Cross-correlation between two Nd arrays
@@ -474,6 +522,7 @@ def xcorr_1d_loop(w1, w2):
     c = np.zeros(w1.shape)
     for i, (wave1, wave2) in enumerate(zip(normalize(w1,0).T, normalize(w2,0).T)):
         c[:,i] = np.correlate(wave1, wave2, mode='same')
+    c[np.isnan(c)]=0 # replace nans with 0 correlation
     return c
 
 def xcorr_2d(w1, w2):
@@ -490,7 +539,9 @@ def normalize(x, axis=0):
     """
     Vanilla normalization (center, reduce) along specified axis
     """
-    return (x-np.mean(x, axis=axis))/np.std(x, axis=axis)
+    s = np.std(x, axis=axis)
+    m = np.mean(x, axis=axis)
+    return (x-m)/s
 
 def get_bins(cwin, cbin):
     mod2=(cwin/cbin)%2
@@ -768,7 +819,7 @@ def _index_of(arr, lookup):
     # Equivalent of np.digitize(arr, lookup) - 1, but much faster.
     lookup = np.asarray(lookup, dtype=np.int32)
     m = (lookup.max() if len(lookup) else 0) + 1
-    tmp = np.zeros(m + 1, dtype=np.int)
+    tmp = np.zeros(m + 1, dtype=np.int64)
     # Ensure that -1 values are kept.
     tmp[-1] = -1
     if len(lookup):
@@ -792,16 +843,14 @@ __all__ = [
 
 
 def _datacheck_peakdetect(x_axis, y_axis):
+    y_axis = np.array(y_axis).ravel()
     if x_axis is None:
-        x_axis = range(len(y_axis))
+        x_axis = np.arange(0, len(y_axis))
 
     if len(y_axis) != len(x_axis):
         raise ValueError(
                 "Input vectors y_axis and x_axis must have same length")
 
-    #needs to be a numpy array
-    y_axis = np.array(y_axis)
-    x_axis = np.array(x_axis)
     return x_axis, y_axis
 
 
@@ -1643,4 +1692,4 @@ def zero_crossings_sine_fit(y_axis, x_axis, fit_window = None, smooth_window = 1
         true_crossings.append(popt[0])
 
 
-    return true_crossings
\ No newline at end of file
+    return true_crossings
